// Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>, see README.TXT for details

#include "globals.hh"
#include "pingutilemap.hh"

PinguTileMap::PinguTileMap(PLF* data, CL_ResourceManager* resource)
{
  // The function should be more flexible to accept different width
  // and heights.

  // Resize the vectors, so that there is enough space for the maps
  map.resize(15);
  for(int i=0; i < map.size(); ++i) {
    map[i].resize(20);
  }

  load(data->get_fg(), resource);
}

PinguTileMap::~PinguTileMap()
{
}

void
PinguTileMap::mark_dirty(int x, int y, int w, int h)
{
  // This border checker is not very flexible
  if (x > 800 || x < 0 || y < 0 | y > 600)
      return;
  
  for(int j=y;  j <= y+h+32; j+=32)
    for(int i=x; i <= x+w+32; i+=32)
      map[j/32][i/32].set_redraw(true);
}

int
PinguTileMap::load(const char* filename, CL_ResourceManager* resource)
{
  int tmp;

  ifstream in(filename);

  if (!in) {
    perror(filename);
    exit(EXIT_FAILURE);
  }

  for(int y=0; y < map.size(); ++y) {
    for(int x=0; x < map[y].size(); ++x) {
      in >> tmp;
      map[y][x].load(tmp, resource);
    }
  }
  return 0;
}

int
PinguTileMap::load(ifstream* in)
{
  int w,h;

  (*in) >> w >> h;

  map.resize(h);
  
  for(int i=0; i < map.size(); ++i)
    map[i].resize(i);
}

void
PinguTileMap::draw_offset(int of_x, int of_y)
{
  for(int y=0; y < map.size(); ++y) {
    for(int x=0; x < map[y].size(); ++x) {
      if (use_dirty_tiles && map[y][x].needs_redraw()) {
	map[y][x].draw(x*32 + of_x, y*32 + of_y);
      } else if (!use_dirty_tiles) {
	if (x >= 0 && x <= 20 && y >=0 && y <= 15) 
	  map[y][x].draw(x*32 + of_x, y*32 + of_y);
      }
    }
  }
}

void
PinguTileMap::start_anim()
{
}

void
PinguTileMap::stop_anim()
{
}

/* EOF */
