// Copyright (C) 1998 Ingo Ruhnke <grumbel@gmx.de>, see README.TXT for details

#include <vector>
#include <cstdio>
#include <ctime>

#include "pingubmpmap.hh"
#include "background.hh"
#include "world.hh"

//CL_Surface* World::gfx_bg = 0;
CL_ResourceManager* World::resource;

World::World()
{

}

World::~World()
{
  delete map;
  delete  bg;
}

void 
World::draw_pingus()
{
  for(int i=0; i < pingu.size(); ++i) {
    pingu[i]->draw_offset(x_offset, y_offset);

    map->mark_dirty(pingu[i]->get_x()-16, pingu[i]->get_y()-32,32,32);
    bg->mark_dirty(pingu[i]->get_x()-16,  pingu[i]->get_y()-32,32,32);
  }  
}

// draw the basic background pattern
void 
World::draw_background()
{
  bg_is_drawn = true;
  bg->draw();
  /*  if (!gfx_bg)
    return;

  for(int y=0; y < 600; y += gfx_bg->get_provider()->get_height())
    for(int x=0; x < 800; x += gfx_bg->get_provider()->get_width())
      gfx_bg->put_screen(x,y);
  */
}

void 
World::draw_playground()
{
  // Draw the exit
  for(int i=0; i < exit.size(); ++i) {
    exit[i]->draw_offset(x_offset, y_offset);
  }
  
  // Draw the gfx map
  pg_is_drawn = true;
  if (map) map->draw_offset(x_offset, y_offset);

  // Draw the entrances
  for(int i=0; i < entrance.size(); ++i) {
    entrance[i]->draw_offset(x_offset, y_offset);
  }

  // Draw the hotspots
}


void 
World::draw_foreground()
{
  fg_is_drawn = true;
}

// Merge the different layers on the screen together
void 
World::draw_offset(int x, int y)
{
  x_offset = x;
  y_offset = y;
  
  draw_background();
  draw_playground();
  draw_pingus();
  draw_foreground();
  
  /*
  static int count;

  // Sets how often the screen should be readrawn
  
  if (++count > 1) { 
    bg_is_drawn = pg_is_drawn = fg_is_drawn = false;
    count = 0;
  }
  
  
  if (!bg_is_drawn) draw_background();
  draw_pingus();
  if (!pg_is_drawn) draw_playground();
  if (!fg_is_drawn) draw_foreground();
  */
}

void 
World::let_move()
{
  // Create new pingus, if enough time is passed
  for(int i=0; i < entrance.size(); ++i) {
    if (entrance[i]->pingu_ready()) {
      pingu.push_back(entrance[i]->get_pingu());
    }
  }
  
  // Let all pingus move
  for(int i=0; i < pingu.size(); ++i) {
    pingu[i]->let_move();
  }

  // Let the pingus catch each other
  

  // Let the traps catch the pingus
  // Traps are not ready

  // Let the exit catch the pingus
  for(int j=0; j < exit.size(); ++j) {
    for(int i=0; i < pingu.size(); ++i) {
      exit[j]->catch_pingu(pingu[i]);
    }
  }
}

void 
World::save()
{
  // Empty should save the state of the wolrd 
}

void 
World::load()
{
  // Empty should load the state of the world
}

void
World::load_data(CL_ResourceManager* res)
{
  resource = res;
}

void 
World::init(PLF* level_data)
{
  //  gfx_bg = CL_Res_Surface::load(level_data->get_bg(), *resource);
  colmap.load(level_data->get_mapfile());

  // load the foreground map
  if (level_data->map_type() == TILE)
    map = new PinguTileMap(level_data, resource);
  else
    map = new PinguBmpMap(level_data->get_fg(), resource);
  
  // load the background map
  bg = new Background(level_data->get_bg(), resource);

  fg_is_drawn = false;
  pg_is_drawn = false;
  bg_is_drawn = false;
  
  int pal;
  srand(time(NULL));

  // Creating Exit and Entrance
  exit.push_back(new Exit(482, 280));
  entrance.push_back(new Entrance(50, 50));

  
  // Setting Pingus at random positions
  pingu.resize(10);

  for(int i=0; i < pingu.size(); ++i) {
    pingu[i] = new Pingu();
    pingu[i]->set_pos( rand() % 800, rand() % 300);
  }
  

  puts("World init : loading gfx");

  Pingu::set_colmap(&colmap);
  Pingu::set_map(map);

  puts("World init : finished");
}

vector<Pingu* >*
World::get_pingu_p()
{
  return &pingu;
}

/* EOF */
